#!/usr/bin/perl -w

# ####################################################### #
# watch_defconfig.pl                                      #
# Configuration utility to build the default config       #
# for spacewatcher.                                       #
# Wayne Pascoe <wayne@penguinpowered.org.uk>              #
#                                                         #
# ####################################################### #

use strict;
use DB_File;
use vars qw(
	    $WORKING_DIR
	    $CONFIG_DIR
	    %DF_SNAPSHOT
	    %PART_CONFIG
	    %MAIL_CONFIG
	    @df_result
	    $partition
	    $used_space
	    $yesnoflag
	    );

init_vars();
do_part_config();
untie(%PART_CONFIG);
do_mail_config();
untie(%MAIL_CONFIG);
` echo "*/10 * * * 1-7 /usr/local/bin/spacewatcher/watchspace.pl > /dev/null 2>&1" >> /var/spool/cron/root`;
do_cron_restart();
confirm_config();

exit(0);

# SUBROUTINES FROM HERE ON!
sub init_vars() {

	my $df_line;
	my $df_count;
	my $df_newline;
	my $df_used;
	my $df_partition;
	
	$WORKING_DIR = "/usr/local/bin/spacewatcher";
	$CONFIG_DIR = "/etc/spacewatcher";
	@df_result = `df -v`;

	foreach $df_line (@df_result) {
                $df_count ++;
                chomp $df_line;
                if ($df_line !~ /\%/ && $df_count > 1) {
                        my $df_newline = ($df_result[$df_count]);
                        $df_line = $df_line . $df_newline;
                        chomp $df_line;
                }
                $df_line =~ tr/a-zA-Z0-9:_\.\//|/cs;
                (undef, undef, undef, undef, $df_used, $df_partition) =
                  split(/\|/, $df_line);
                if (substr($df_partition,0,1) eq "/") {
                        $DF_SNAPSHOT{$df_partition} = $df_used;
                }
        }

}

sub do_part_config() {

	my $response;
	my $perc_response;
	my $sure;

	$yesnoflag = "NOTSET";
	
	tie (%PART_CONFIG, "DB_File", "$CONFIG_DIR/partitions.db") or die "Can't open partitions.db.. $!\n";

	foreach $partition (keys %DF_SNAPSHOT) {
		$PART_CONFIG{$partition} = 90;
	}
}

sub do_mail_config() {

	my $response;
	my $perc_response;
	my $sure;
	my $From_address;
	my $To_address;
	my $Host_name = `/bin/hostname`;
	
	tie (%MAIL_CONFIG, "DB_File", "$CONFIG_DIR/mail_config.db") or die "Can't open mail_config.db.. $!\n";

	$MAIL_CONFIG{'from'} = "spacewatcher\@$Host_name";
	$MAIL_CONFIG{'to'} = "root\@$Host_name";
}

sub confirm_config() {

	my $part;
	my $from_mail;
	my $to_mail;

	tie (%PART_CONFIG, "DB_File", "$CONFIG_DIR/partitions.db") or die "Can't open partitions.db.. $!\n";
	tie (%MAIL_CONFIG, "DB_File", "$CONFIG_DIR/mail_config.db") or die "Can't open mail_config.db.. $!\n";

	chomp $MAIL_CONFIG{'from'};
	chomp $MAIL_CONFIG{'to'};

	print("I will be monitoring the following partitions and alerting you\n");
	print("the amount of space used exceeds the threshold stated.\n\n");
	print("Partition\t\t\tThreshold\n");
	print("---------\t\t\t---------\n");	
	foreach $part(keys %PART_CONFIG) {
		if ( length($part) < 7 ) {
			print("$part\t\t\t\t$PART_CONFIG{$part}\n");
		} else {
			print("$part\t\t\t$PART_CONFIG{$part}\n");
		}
	}
	print("\n\n");
	print("All mails will have a From field of $MAIL_CONFIG{'from'}\n");
	print("They will be sent to $MAIL_CONFIG{'to'}\n\n");
	print("If you want to change any of these settings (partitions or ");
	print("e-mail address') \nthen please run \n");
	print("/usr/local/bin/spacewatcher/configure.pl\n\n");
	print("You also need to know about the line I've added to cron. I've\n");
	print("added it as root for now, but if you want to change permissions\n");
	print("on the files in /etc/spacewatcher and /usr/local/bin/spacewatcher\n");
	print("then you can run it as anyone you like. Have fun!\n");

}

sub do_cron_restart() {
	
	my $status = `/etc/rc.d/init.d/crond status`;
	if ($status =~ /running/) {
		`/etc/rc.d/init.d/crond stop`;
		sleep 1;
		`/etc/rc.d/init.d/crond start`;
	} elsif ($status =~ /Resource temporarily unavailable/) {
		`/etc/rc.d/init.d/crond stop`;
                sleep 1;
                `/etc/rc.d/init.d/crond start`;
	}
}
