#!/usr/bin/perl -w

# ####################################################### #
# configure.pl                                            #
# Configuration utility for spacewatcher                  #
# Wayne Pascoe <wayne@penguinpowered.org.uk>              #
#                                                         #
# ####################################################### #

use strict;
use DB_File;
use vars qw(
	    $WORKING_DIR
	    $CONFIG_DIR
	    %DF_SNAPSHOT
	    %PART_CONFIG
	    %MAIL_CONFIG
	    @df_result
	    $partition
	    $used_space
	    $yesnoflag
	    );

init_vars();
print_table();
do_part_config();
untie(%PART_CONFIG);
do_mail_config();
untie(%MAIL_CONFIG);
confirm_conf();

exit(0);

# SUBROUTINES FROM HERE ON!
sub init_vars() {

	my $df_line;
	my $df_count;
	my $df_newline;
	my $df_used;
	my $df_partition;
	
	$WORKING_DIR = "/usr/local/bin/spacewatcher";
	$CONFIG_DIR = "/etc/spacewatcher";
	@df_result = `df -v`;

	foreach $df_line (@df_result) {
                $df_count ++;
                chomp $df_line;
                if ($df_line !~ /\%/ && $df_count > 1) {
                        my $df_newline = ($df_result[$df_count]);
                        $df_line = $df_line . $df_newline;
                        chomp $df_line;
                }
                $df_line =~ tr/a-zA-Z0-9:_\.\//|/cs;
                (undef, undef, undef, undef, $df_used, $df_partition) =
                  split(/\|/, $df_line);
                if (substr($df_partition,0,1) eq "/") {
                        $DF_SNAPSHOT{$df_partition} = $df_used;
                }
        }

}

sub print_table() {
	
	print "\nYour existing Partition list and usage is as follows :\n";
	print "-------------------------------------------------------\n\n";
	foreach $partition (keys %DF_SNAPSHOT) {
		print "$partition\t\t\t$DF_SNAPSHOT{$partition}\%\n";
	}
	print "\n-------------------------------------------------------\n\n";
	print "We will now present you with your partitions, one at a time\n";
	print "and ask if you want that partition monitored. If you do, \n";
	print "you will then be asked to set a maximum threshold. Once the\n";
	print "amount of used space on a partition exceeds this, you will\n";
	print "be notified.\n\n";

}

sub do_part_config() {

	my $response;
	my $perc_response;
	my $sure;

	$yesnoflag = "NOTSET";
	
	tie (%PART_CONFIG, "DB_File", "$CONFIG_DIR/partitions.db") or die "Can't open partitions.db.. $!\n";

	foreach $partition (keys %DF_SNAPSHOT) {
		print "Would you like to monitor your $partition partition?    [YES/no] : ";
		$response = <>;
		chomp($response);
		$response = uc($response);

		if ( $response eq "NO" || $response eq "N" ) {
			if ($PART_CONFIG{$partition}) {
				delete $PART_CONFIG{$partition};
			}
			
			if ($yesnoflag eq "NOTSET") {
				$yesnoflag = "NO";
			}

		} elsif ($response eq "YES" || $response eq "Y" || $response eq "") {
			$yesnoflag = "YES";
			print "At what usage percentage would you like to be notified?  [95] : ";
			$perc_response = <>;
			chomp $perc_response;
			if ($perc_response eq "") { $perc_response = "95" };
			if ($perc_response < $DF_SNAPSHOT{$partition}) {
				print "\nYou have specified a threshold that is lower than your current\n";
				print "usage... If you leave this like it is, you will not get efficient use\n";
				print "from spacewatcher... Are you sure you want this?     [yes/NO] : ";
				$sure = <>;
				chomp $sure;
				$sure = uc($sure);
				if (($sure eq "YES") || ($sure eq "Y")) {
					$PART_CONFIG{$partition} = $perc_response;
				} elsif (($sure eq "NO" || $sure eq "" || $sure eq "N") && $PART_CONFIG{$partition} ) {
					delete $PART_CONFIG{$partition};
				}
			} else {
				$PART_CONFIG{$partition} = $perc_response;
			}
		}
	}
}

sub do_mail_config() {

	my $response;
	my $perc_response;
	my $sure;
	my $From_address;
	my $To_address;
	
	if ($yesnoflag ne "NO") {
		tie (%MAIL_CONFIG, "DB_File", "$CONFIG_DIR/mail_config.db") or die "Can't open mail_config.db.. $!\n";

		print "\n\nOk, so we know that you want to know what we know about your partitions...\n";
		print "Now it would _really_ help us if we knew where to reach you and who\n";
		print "you would like us to be... \nThis is where you enter <To> and <From> e-mail ";
		print "address'.\n\n The <From> address is who the mail will appear to come from. An \n";
		print "example of this would be alerter\@foo.bar\n";
		print "The <To> address is an e-mail address that you want to \n";
		print "be alerted when space thresholds are reached.\n\n";
		print "Please enter a single mail address for the <From> field of any alert e-mails...\n";
		print "From Address : ";
		do {
			$From_address = <>;
			chomp ($From_address);
			if ($From_address eq "") {
				print "You didn't enter an address... Please try again!\n";
			}
		} while ($From_address eq "");
		$MAIL_CONFIG{'from'} = $From_address;

		print "\nThank you. Now please enter an e-mail address to be \n";
		print "notified when thresholds are met / exceeded. For example\n";
		print "me\@somewhere.com\n";
		print "To Address : ";
		do {
			$To_address = <>;
			chomp ($To_address);
			if ($To_address eq "") {
				print "You didn't enter any e-mail address'... Please try again!\n";
			}
		} while ($To_address eq "");
		$MAIL_CONFIG{'to'} = $To_address;
	}
}

sub confirm_conf() {
	
	tie (%PART_CONFIG, "DB_File", "$CONFIG_DIR/partitions.db") or die "Can't open partitions.db.. $!\n";
	tie (%MAIL_CONFIG, "DB_File", "$CONFIG_DIR/mail_config.db") or die "Can't open mail_config.db.. $!\n";

	print "\n\n\nRight then... Let's review, shall we? \n";
	print "You would like us to monitor the following partitions and warn you when\n";
	print "the percentage of space used exceeds the number specified...\n\n";
	print "Partition\t\t\tThreshold\n";
	print "---------\t\t\t---------\n";
	foreach $partition (keys %PART_CONFIG) {
		if (length($partition) < 7) {
			print "$partition\t\t\t\t$PART_CONFIG{$partition}\n";
		} else {
			print "$partition\t\t\t$PART_CONFIG{$partition}\n";
		}
	}
	print "\n";
	print "When we mail you alerts, you want the mail to appear from \n";
	print "$MAIL_CONFIG{'from'}\n\n";
	print "Those alerts should be e-mailed to \n";
	print "$MAIL_CONFIG{'to'}\n\n";
	print "All this information has been written out to the relevant db files.\n";
	print "If any of it is wrong, or you have changed your mind about address\n";
	print "details, please run the configure.pl script again.\n";
	print "Thanks for playing...\n\n";
	

	untie (%PART_CONFIG);
	untie (%MAIL_CONFIG);
}
