#!/usr/bin/perl -w

# ########################################################### #
# watchspace.pl                                               #
# Watch space on a machine and send an e-mail / sms if any    #
# partition's free space falls below a threshold.             #
# Wayne Pascoe <wayne@penguinpowered.org.uk>                  #
# ########################################################### #

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

use strict;
use DB_File;
use vars qw(
	    %PARTITIONS
	    %DF_SNAPSHOT
	    %FULL
	    %MAIL
	    @EMAILS
	    @df_result
	    $WORKINGDIR
	    $CONFIG_DIR
	    $line
	    $use
	    $partition
	    $message
	    $host
	    );

init_vars();
test_space();
untie(%FULL);
untie(%PARTITIONS);
untie(%MAIL);


# ##############################################################
#    SUBROUTINES FROM HERE ON
# ##############################################################

#
# test_space -> Check the space of the df_result against the 
# partitions in the PARTITIONS hash
#
sub test_space() {

	my $current_partition;
	my $list_partition;
	my $recipient;

	foreach $list_partition (keys %PARTITIONS) {
#		print "Testing $list_partition... should be $PARTITIONS{$list_partition}\n";
		if (exists $DF_SNAPSHOT{$list_partition}) {
			if ($PARTITIONS{$list_partition} < $DF_SNAPSHOT{$list_partition}) {
				if (!$FULL{$list_partition}) {
					$message = "ALERT from $host . Not enough space on $list_partition\n";
					panick_mail($MAIL{'to'}, 'Space problem!',  $message);
					$FULL{$list_partition} = $PARTITIONS{$list_partition};
				}
			} elsif ($PARTITIONS{$list_partition} > $DF_SNAPSHOT{$list_partition}) {
				if ($FULL{$list_partition}) {
					delete $FULL{$list_partition};
					$message = "SPACE OK! on $host. Space is healthy again on $list_partition\n";
					panick_mail($MAIL{'to'}, 'Space problem gone!', $message);
				}
			}
		}
	}
}

#
# init_vars -> Initialise all variables for use throughout the 
# rest of the script
#
sub init_vars() {

	my $line;
	my $used;
	my $partition;
	my $count = 0;
	my $df_command;
	my $platform;

	$host = `hostname`;
	chomp $host;
	$WORKINGDIR = "/usr/local/bin/spacewatcher";
	$CONFIG_DIR = "/etc/spacewatcher";

#
# Get disk info from df and put it into useable variables
#
	$platform = `uname`;
	chomp $platform;
	if ($platform eq 'Linux') {$df_command = "df -v";} elsif
        ($platform eq 'SunOS') {$df_command = "df -k";} elsif 
	($platform eq 'FreeBSD') {$df_command = "df -v";} elsif
	($platform eq 'SCO_SV') {$df_command = "df -v";} else
	{$df_command = "df -v";}

	@df_result = `$df_command`;
	
	foreach $line (@df_result) {
		$count ++;
		chomp $line;
		if ($line !~ /\%/ && $count > 1) {
			my $newline = ($df_result[$count]);
			$line = $line . $newline;
			chomp $line;
		}
		$line =~ tr/a-zA-Z0-9:_\.\//|/cs;
		(undef, undef, undef, undef, $used, $partition) =
	   	  split(/\|/, $line);
		if (substr($partition,0,1) eq "/") {
                       	$DF_SNAPSHOT{$partition} = $used;
               	}
	}

#
# Here we set the maximum capacity before panicking. This is the
# % used figure from df. If the df value for use% exceeds the 
# value stated here, we will panick!
#
  	tie (%PARTITIONS, "DB_File", "$CONFIG_DIR/partitions.db") or die "Can't open partitions.db.. $!\n";

#
# List of e-mails to be notified when things go pear-shaped
#
  	tie (%MAIL, "DB_File", "$CONFIG_DIR/mail_config.db") or die "Can't open mail_config.db.. $!\n";
	
#
# Get a list of partitions that we know to be full and have already
# bugged someone about.
#
  	tie (%FULL, "DB_File", "$WORKINGDIR/full.db") or die "Can't open full.db.. $!\n";

}

# Send the panicking e-mail if all is pear shaped...
sub panick_mail {

	if (@_ ne "3") { die "Please call panick_mail with the following 3 variables...
		              Recipient to send to 
			      Subject of message 
			      Message to send."; }
	my $recipient = shift;
	my $subject = shift;
	my $message = shift;
	
	open (MAIL,'|/usr/lib/sendmail ' . $recipient) or die "Could not send mail...$!\n";
	print (MAIL "From: $MAIL{'from'}\n");
	print (MAIL "Subject: $subject\n\n");
	print (MAIL $message);
	close (MAIL);
	return('SUCCESS');
}
